/* nntp.c - NNTP routines used to talk to a NNTP server
   Copyright (C) 2000 Rik Snel <snel@phys.uu.nl>
   Parts copyright by Eric S. Raymond and the Free Software Foundation

   This file is part of newsleech.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
*/
#include<stdio.h>
#include<unistd.h>
#include<varargs.h>
#include<string.h>

#include "config.h"
#include "newsleech.h"
#include "nntp.h"
#include "tcp.h"

void server_said(char *status) {
	if (!(verbose & 1)) fprintf(stderr, "The server said: %s\n", status);
	return;
}

int NNTP_status(int sock, char *status) {
	int length;
	length = tcp_socket_readline(sock, status, 2048);
	if (status[strlen(status)-1] == '\n') status[strlen(status)-1] = '\0';
	if (status[strlen(status)-1] == '\r') status[strlen(status)-1] = '\0';
	if (status[length-2] == '\r') status[length-1] = '\0';
	if (verbose & 1) fprintf(stderr, "NNTP< %s\n", status);
	if ((status[0] == '1' || status[0] == '2' || status[0] == '3') && status[3] == ' ') return(0);
	return(-1);
}

int NNTP_QUIT(int sock) {
	char status[2048];
	tcp_socket_printf(sock, "QUIT\r\n");
	if (verbose & 1) fprintf(stderr, "NNTP> QUIT\n");
	if (NNTP_status(sock, status) < 0) {
		server_said(status);
		return(-1);
	}
	close_tcp_connection(sock);
	return(0);
}

/* This function catches all data from a command that ends
 * in '.\r\n' It returns -1 if the server gives an error if
 * the request immediately fails and it returns -2 if there is a read
 * error.
 */
int NNTP_ARTICLE_and_friends(int sock, char *command, FILE *fp) {
	int length;
	char status[2048], buf[8192];
	tcp_socket_printf(sock, "%s\r\n", command);
	if (verbose & 1) fprintf(stderr, "NNTP> %s \n", command);
	if (NNTP_status(sock, status) < 0) {
		server_said(status);
		return(-1);
	}
	do {
		length = tcp_socket_readline(sock, buf, sizeof(buf));
		if (length <= 0) {
			fprintf(stderr, "Read error.\n");
			return(-2);
		}
		if (buf[0] == '.' && buf[1] == '.') fprintf(fp, "%s", buf+1);
		else {
			if (!(buf[0] == '.' && buf[1] == '\r' && buf[2] == '\n')) fprintf(fp, "%s", buf);
			else break;
		}
	} while (1);
	return(0);
}

int NNTP_AUTHINFO(int sock, char *user, char *pass) {
	char status[2048];
	tcp_socket_printf(sock, "AUTHINFO user %s\r\n", user);
	if (verbose & 1) fprintf(stderr, "NNTP> AUTHINFO user %s\r\n", user);
	if (NNTP_status(sock, status) < 0) {
		fprintf(stderr, "Error, to be implemented.\n");
		return(-1);
	}
	if (pass != NULL) {
		tcp_socket_printf(sock, "AUTHINFO pass %s\r\n", pass);
		if (verbose & 1) fprintf(stderr, "NNTP> AUTHINFO pass xxxxxxxx\r\n");
		if (NNTP_status(sock, status) < 0) {
			fprintf(stderr, "Error, to be implemented.\n");
			return(-1);
		}
	}
	return(0);
}

int NNTP_GROUP(int sock, struct groupinfo *group) {
	char status[2048];
	int junk;
	tcp_socket_printf(sock, "GROUP %s\r\n", group->name);
	if (verbose & 1) fprintf(stderr, "NNTP> GROUP %s\r\n", group->name);
	if (NNTP_status(sock, status) < 0) {
		if (status[1] == '1' && status[2] == '1') {
			server_said(status);
		} else {
			server_said(status);
		}
		return(-1);
	}
	sscanf(status, "%d %d %d %d", &junk, &group->total, &group->first, &group->last);
	return(0);
}

int NNTP_init(struct serverinfo *server) {
	int sock;
	char status[2048];
	while (1) {
		if ((sock = open_tcp_connection(server->name, server->port)) < 0) return(-2);
		if (NNTP_status(sock, status) < 0) {
			if ((status[0] == '5' && status[1] == '0' && status[2] == '2') || (status[0] == '4' && status[1] == '0' && status[2] == '0')) {
				server_said(status);
				close_tcp_connection(sock);	
				if (server->interval == 0) return(-1);
				fprintf(stderr, "Retry in %d second(s).\n", server->interval);
			} else {
				fprintf(stderr, "The server returned an unknown error.\n");
				close_tcp_connection(sock);
				return(-1);
			}
		} else {
			if (server->user != NULL) {
				NNTP_AUTHINFO(sock, server->user, server->pass);
			}		
			return(sock);
		}
	sleep(server->interval);
	}
}
